using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace Stunnware.Crm.Tools {
   /// <summary>
   /// Summary description for FormException.
   /// </summary>
   public class ExceptionBox : System.Windows.Forms.Form {

      private System.Windows.Forms.Label labelMessage;
      private System.Windows.Forms.Label labelHeader;
      private System.Windows.Forms.Label labelMessageTextHeader;
      private System.Windows.Forms.Button buttonDetails;
      private System.Windows.Forms.Button buttonOk;
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;
      private System.Windows.Forms.Label labelDivider;
      private System.Windows.Forms.Label labelDetailsHeader;
      private System.Windows.Forms.TextBox textBoxException;

      private Exception exception;
      private bool      showDetails = false;

      /// <summary>
      /// Displays the exception form.
      /// </summary>
      /// <param name="owner">The owner form.</param>
      /// <param name="exception">The exception to display.</param>
      public static void Show(Form owner, Exception exception) {
         Show(owner, exception, null);
      }

      /// <summary>
      /// Displays the exception form.
      /// </summary>
      /// <param name="owner">The owner form.</param>
      /// <param name="exception">The exception to display.</param>
      /// <param name="headerText">A specific header text to show in the dialog. Set to null or an empty string if you want to use the default header.</param>
      public static void Show(Form owner, Exception exception, string headerText) {
         ExceptionBox form = new ExceptionBox(owner, exception, headerText);
         form.ShowDialog(owner);
      }

      private ExceptionBox(Form owner, Exception exception, string headerText) {
         
         InitializeComponent();

         this.Text = (owner != null) ? owner.Text : string.Empty;
         this.exception = exception;

         if ((headerText != null) && (headerText != String.Empty)) {
            this.labelHeader.Text = headerText;
         }

         string exceptionMessage = exception.Message;

         if (exception is System.Web.Services.Protocols.SoapException) {
            exceptionMessage += "\r\n:Details: " + (exception as System.Web.Services.Protocols.SoapException).Detail.InnerXml;
         }

         else if (exception is System.Net.WebException) {
            exceptionMessage += "\r\n:Status: " + (exception as System.Net.WebException).Status;
         }

         Graphics g = this.CreateGraphics();
         SizeF messageSize = g.MeasureString(exceptionMessage, labelMessage.Font, labelMessage.Width);

         int height = (int) messageSize.Height + 8;
         height = (height >> 3) << 3;

         labelMessage.Height = height + 16;
         labelDetailsHeader.Top = 112 + labelMessage.Height - 24;
         textBoxException.Top = 136 + labelMessage.Height - 24;

         labelMessage.Text = exceptionMessage;
         textBoxException.Text = exception.ToString();

         AdjustDialogHeight();
         this.buttonOk.Focus();
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ExceptionBox));
         this.labelMessage = new System.Windows.Forms.Label();
         this.labelHeader = new System.Windows.Forms.Label();
         this.labelMessageTextHeader = new System.Windows.Forms.Label();
         this.buttonDetails = new System.Windows.Forms.Button();
         this.labelDivider = new System.Windows.Forms.Label();
         this.buttonOk = new System.Windows.Forms.Button();
         this.labelDetailsHeader = new System.Windows.Forms.Label();
         this.textBoxException = new System.Windows.Forms.TextBox();
         this.SuspendLayout();
         // 
         // labelMessage
         // 
         resources.ApplyResources(this.labelMessage, "labelMessage");
         this.labelMessage.MaximumSize = new System.Drawing.Size(456, 400);
         this.labelMessage.Name = "labelMessage";
         // 
         // labelHeader
         // 
         resources.ApplyResources(this.labelHeader, "labelHeader");
         this.labelHeader.Name = "labelHeader";
         // 
         // labelMessageTextHeader
         // 
         resources.ApplyResources(this.labelMessageTextHeader, "labelMessageTextHeader");
         this.labelMessageTextHeader.Name = "labelMessageTextHeader";
         // 
         // buttonDetails
         // 
         this.buttonDetails.Cursor = System.Windows.Forms.Cursors.Hand;
         resources.ApplyResources(this.buttonDetails, "buttonDetails");
         this.buttonDetails.Name = "buttonDetails";
         this.buttonDetails.Click += new System.EventHandler(this.buttonDetails_Click);
         // 
         // labelDivider
         // 
         this.labelDivider.BackColor = System.Drawing.SystemColors.WindowText;
         resources.ApplyResources(this.labelDivider, "labelDivider");
         this.labelDivider.Name = "labelDivider";
         // 
         // buttonOk
         // 
         this.buttonOk.Cursor = System.Windows.Forms.Cursors.Hand;
         this.buttonOk.DialogResult = System.Windows.Forms.DialogResult.Cancel;
         resources.ApplyResources(this.buttonOk, "buttonOk");
         this.buttonOk.Name = "buttonOk";
         this.buttonOk.Click += new System.EventHandler(this.buttonOk_Click);
         // 
         // labelDetailsHeader
         // 
         resources.ApplyResources(this.labelDetailsHeader, "labelDetailsHeader");
         this.labelDetailsHeader.Name = "labelDetailsHeader";
         // 
         // textBoxException
         // 
         resources.ApplyResources(this.textBoxException, "textBoxException");
         this.textBoxException.Name = "textBoxException";
         this.textBoxException.ReadOnly = true;
         // 
         // ExceptionBox
         // 
         this.AcceptButton = this.buttonOk;
         resources.ApplyResources(this, "$this");
         this.CancelButton = this.buttonOk;
         this.Controls.Add(this.textBoxException);
         this.Controls.Add(this.labelDetailsHeader);
         this.Controls.Add(this.buttonOk);
         this.Controls.Add(this.labelDivider);
         this.Controls.Add(this.buttonDetails);
         this.Controls.Add(this.labelMessageTextHeader);
         this.Controls.Add(this.labelHeader);
         this.Controls.Add(this.labelMessage);
         this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
         this.MaximizeBox = false;
         this.MinimizeBox = false;
         this.Name = "ExceptionBox";
         this.ResumeLayout(false);
         this.PerformLayout();

      }
      #endregion

      private void AdjustDialogHeight() {

         int addHeight = showDetails ? (376 - 190) : 0;

         this.Height = 190 + labelMessage.Height - 24 + addHeight;
         labelDivider.Top = 104 + labelMessage.Height - 24 + addHeight;
         buttonDetails.Top = 120 + labelMessage.Height - 24 + addHeight;
         buttonOk.Top = buttonDetails.Top;

         labelDetailsHeader.Visible = showDetails;
         textBoxException.Visible = showDetails;
      }

      private void buttonDetails_Click(object sender, System.EventArgs e) {         
         showDetails = !showDetails;
         AdjustDialogHeight();
      }

      private void buttonOk_Click(object sender, System.EventArgs e) {
         this.DialogResult = DialogResult.OK;
         this.Close();
      }
   }
}