﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Stunnware.Tools.CrmService3;
using System.Net;
using System.Web.Services.Protocols;

namespace Stunnware.Crm.Tools {

   /// <summary>
   /// This form is responsible to execute your request and to display the SOAP messages being sent
   /// to the server and received from the server. It also converts the SOAP request document to the
   /// appropriate JavaScript code.
   /// </summary>
   public partial class Form1 : Form {
      public Form1() {
         InitializeComponent();
      }

      /// <summary>
      /// Starts the execution of a web service. Put your code inside the try block. You can use 
      /// every web service you like, so you're not limited to query expressions. 
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void buttonStart_Click(object sender, EventArgs e) {

         buttonStart.Enabled = false;
         textBoxRequest.Text = string.Empty;
         textBoxResponse.Text = string.Empty;

         try {
            //TODO: Replace the content of the try block with the code you want to execute
            //in JavaScript
            QueryExpression query = new QueryExpression();
            query.ColumnSet = new AllColumns();
            query.EntityName = EntityName.account.ToString();

            FilterExpression filter = new FilterExpression();
            query.Criteria = filter;

            ConditionExpression hasEmailAddress = new ConditionExpression();
            filter.Conditions = new ConditionExpression[] { hasEmailAddress };

            hasEmailAddress.AttributeName = "emailaddress1";
            hasEmailAddress.Operator = ConditionOperator.NotNull;

            //TODO: Change the server name "stunnwarecrm" to the name or ip address of your Crm
            //Server machine.
            //TODO: If you want to update the web reference, you must also change the server name 
            //in the CrmService3 web reference.
            //TODO: If you receive an authentication error when trying to connect to the web service,
            //you should try to use a NetworkCredential instead of the DefaultCredentials.
            CrmLogService service = new CrmLogService();
            service.Url = "http://stunnwarecrm/mscrmservices/2006/crmservice.asmx";
            service.Credentials = CredentialCache.DefaultCredentials;

            service.SoapMessageSerializationFinished += new SoapMessageEventHandler(service_SoapMessageSerializationFinished);
            service.RetrieveMultiple(query);
         }

         catch (Exception x) {
            ExceptionBox.Show(this, x);
         }

         finally {
            buttonStart.Enabled = true;
         }
      }

      /// <summary>
      /// This event handler is called whenever a SOAP message was sent or retrieved. It populates
      /// the appropriate text fields and creates the JavaScript code.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      void service_SoapMessageSerializationFinished(object sender, SoapMessageEventArgs e) {
        
         if (e.MessageType == SoapMessageType.Request) {
            textBoxRequest.Text = e.FormattedMessageXml;

            CrmLogService service = sender as CrmLogService;

            StringBuilder code = new StringBuilder();
            
            code.AppendLine("var xml = \"\" + ");

            string[] xmlLines = e.FormattedMessageXml.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

            foreach(string s in xmlLines) {
               code.AppendFormat("\"{0}\" + \r\n", s.Replace("\"", "\\\""));
            }

            code.AppendLine("\"\";");
            code.AppendLine();

            code.AppendLine("var xmlHttpRequest = new ActiveXObject(\"Msxml2.XMLHTTP\");");
            code.AppendLine();
            code.AppendLine("xmlHttpRequest.Open(\"POST\", \"/mscrmservices/2006/CrmService.asmx\", false);");
            code.AppendLine("xmlHttpRequest.setRequestHeader(\"SOAPAction\",\"http://schemas.microsoft.com/crm/2006/WebServices/" + e.SoapClientMessage.MethodInfo.Name + "\");");
            code.AppendLine("xmlHttpRequest.setRequestHeader(\"Content-Type\", \"text/xml; charset=utf-8\");");
            code.AppendLine("xmlHttpRequest.setRequestHeader(\"Content-Length\", xml.length);");
            code.AppendLine("xmlHttpRequest.send(xml);");
            code.AppendLine();

            code.AppendLine("var resultXml = xmlHttpRequest.responseXML;");
            code.AppendLine("alert(resultXml.xml);");

            textBoxJavaScript.Text = code.ToString();

            Application.DoEvents();
         }

         else {
            textBoxResponse.Text = e.FormattedMessageXml;
            Application.DoEvents();
         }
      }

      private void buttonCopy_Click(object sender, EventArgs e) {
       
         if (tabControl1.SelectedTab == tabPageRequest) {
            Clipboard.SetText(textBoxRequest.Text);
         }

         else if (tabControl1.SelectedTab == tabPageResponse) {
            Clipboard.SetText(textBoxResponse.Text);
         }

         else if (tabControl1.SelectedTab == tabPageJavaScript) {
            Clipboard.SetText(textBoxJavaScript.Text);
         }
      }
   }
}