﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Web.Services.Protocols;

namespace Stunnware.Crm.Tools {

   /// <summary>
   /// The SoapMessageEventArgs is used to pass the SOAP messages to the main form. It is used in
   /// the SoapMessageEventHandler delegate.
   /// </summary>
   class SoapMessageEventArgs : EventArgs {

      SoapClientMessage _soapClientMessage;
      SoapMessageType _messageType;
      string _message;
      string _formattedMessage;

      /// <summary>
      /// Creates a new instance of this class.
      /// </summary>
      /// <param name="soapClientMessage"></param>
      /// <param name="messageType"></param>
      /// <param name="message"></param>
      public SoapMessageEventArgs(SoapClientMessage soapClientMessage, SoapMessageType messageType, string message) {
         _soapClientMessage = soapClientMessage;
         _messageType = messageType;
         _message = message;
         _formattedMessage = FormatXml(message);
      }

      /// <summary>
      /// The SOAP message being sent or received.
      /// </summary>
      public SoapClientMessage SoapClientMessage {
         get {
            return _soapClientMessage;
         }
      }

      /// <summary>
      /// Formats the passed xml string.
      /// </summary>
      /// <param name="xml"></param>
      /// <returns></returns>
      static string FormatXml(string xml) {

         StringBuilder sb = new StringBuilder();         
         XmlWriterSettings settings = new XmlWriterSettings();

         settings.Indent = true;
         settings.OmitXmlDeclaration = false;

         using (XmlWriter w = XmlTextWriter.Create(sb, settings)) {
            using (TextReader tr = new StringReader(xml)) {
               using (XmlReader r = XmlTextReader.Create(tr)) {
                  w.WriteNode(r, true);
               }
            }
         }

         return sb.ToString();
      }

      /// <summary>
      /// The message type.
      /// </summary>
      public SoapMessageType MessageType {
         get {
            return _messageType;
         }
      }

      /// <summary>
      /// The unformatted XML.
      /// </summary>
      public string MessageXml {
         get {
            return _message;
         }
      }

      /// <summary>
      /// The formatted XML.
      /// </summary>
      public string FormattedMessageXml {
         get {
            return _formattedMessage;
         }
      }
   }
}
